# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import argparse
from knack.util import CLIError
from azure.mgmt.eventgrid.models import (
    StaticDeliveryAttributeMapping,
    DynamicDeliveryAttributeMapping
)

STATIC = "static"
DYNAMIC = "dynamic"


# pylint: disable=protected-access
# pylint: disable=too-few-public-methods
class AddDeliveryAttributeMapping(argparse._AppendAction):
    def __call__(self, parser, namespace, values, option_string=None):
        valuesLen = len(values)
        if valuesLen < 2:
            raise CLIError('usage error: --delivery-attribute-mapping NAME TYPE [SOURCEFIELD] [VALUE] [ISSECRET]')
        name = values[0]
        delivery_attribute_type = values[1]

        if delivery_attribute_type.lower() == STATIC:
            if valuesLen < 3 or valuesLen > 4:
                raise CLIError('usage error: --delivery-attribute-mapping <name> static <value> [<true/false>]')
            value = values[2]
            isSecret = False
            if valuesLen == 4:
                isSecret = bool(values[3])
            delivery_attribute_mapping = StaticDeliveryAttributeMapping(
                name=name,
                type=delivery_attribute_type,
                value=value,
                is_secret=isSecret)
        elif delivery_attribute_type.lower() == DYNAMIC:
            if valuesLen != 3:
                raise CLIError('usage error: --delivery-attribute-mapping <name> dynamic <value>')
            sourceField = values[2]
            delivery_attribute_mapping = DynamicDeliveryAttributeMapping(
                name=name,
                type=delivery_attribute_type,
                source_field=sourceField)
        else:
            raise CLIError('usage error: --delivery-attribute-mapping NAME TYPE [SOURCEFIELD] [VALUE] [ISSECRET]')

        if namespace.delivery_attribute_mapping is None:
            namespace.delivery_attribute_mapping = []
        namespace.delivery_attribute_mapping.append(delivery_attribute_mapping)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def network_client_factory(cli_ctx, **kwargs):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK, **kwargs)


def resource_client_factory(cli_ctx, **_):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)


def cf_application_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).application_gateways


def cf_connection_monitor(cli_ctx, _):
    return network_client_factory(cli_ctx).connection_monitors


def cf_flow_logs(cli_ctx, _):
    return network_client_factory(cli_ctx).flow_logs


def cf_private_link_services(cli_ctx, _):
    return network_client_factory(cli_ctx).private_link_services


def cf_load_balancers(cli_ctx, _):
    return network_client_factory(cli_ctx).load_balancers


def cf_load_balancer_backend_pools(cli_ctx, _):
    return network_client_factory(cli_ctx).load_balancer_backend_address_pools


def cf_local_network_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).local_network_gateways


def cf_network_interfaces(cli_ctx, _):
    return network_client_factory(cli_ctx).network_interfaces


def cf_network_watcher(cli_ctx, _):
    return network_client_factory(cli_ctx).network_watchers


def cf_packet_capture(cli_ctx, _):
    return network_client_factory(cli_ctx).packet_captures


def cf_private_access(cli_ctx, _):
    return network_client_factory(cli_ctx).available_private_access_services


def cf_virtual_network_gateway_connections(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_network_gateway_connections


def cf_virtual_network_gateways(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_network_gateways


def cf_dns_references(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).dns_resource_reference


def cf_dns_mgmt_zones(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).zones


def cf_dns_mgmt_record_sets(cli_ctx, _):
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK_DNS).record_sets


def cf_virtual_router(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_routers


def cf_virtual_hub(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_hubs


def cf_virtual_router_peering(cli_ctx, _):
    return network_client_factory(cli_ctx).virtual_router_peerings



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command_group(
    "policy-insights",
)
class __CMDGroup(AAZCommandGroup):
    """Manage resource policies.

    Azure Policy helps to enforce organizational standards and to assess compliance at-scale. Through its compliance dashboard, it provides an aggregated view to evaluate the overall state of the environment, with the ability to drill down to the per-resource, per-policy granularity. It also helps to bring your resources to compliance through bulk remediation for existing resources and automatic remediation for new resources.
    """
    pass


__all__ = ["__CMDGroup"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "sql mi dtc update",
)
class Update(AAZCommand):
    """Update managed instance DTC settings.

    This command updates the managed instance DTC settings. All parameters are optional.

    :example: Enable DTC on a managed instance
        az sql mi dtc update -g resourceGroup1 --mi managedInstance1 --dtc-enabled true
        az sql mi dtc update --ids /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/ResourceGroup1/providers/Microsoft.Sql/managedInstances/ManagedInstance1/dtc/current --dtc-enabled true

    :example: Allow XA transactions for managed instance DTC
        az sql mi dtc update -g resourceGroup1 --mi managedInstance1 --xa-transactions-enabled true
    """

    _aaz_info = {
        "version": "2022-05-01-preview",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.sql/managedinstances/{}/dtc/{}", "2022-05-01-preview"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.managed_instance_name = AAZStrArg(
            options=["--mi", "--managed-instance-name"],
            help="The name of the managed instance.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            help="Name of the resource group",
            required=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.dtc_enabled = AAZBoolArg(
            options=["--dtc-enabled"],
            arg_group="Properties",
            help="Active status of managed instance DTC.",
            nullable=True,
        )
        _args_schema.external_dns_suffix_search_list = AAZListArg(
            options=["--external-dns-suffixes", "--external-dns-suffix-search-list"],
            arg_group="Properties",
            help="External dns suffix search list of managed instance DTC.",
            nullable=True,
        )

        external_dns_suffix_search_list = cls._args_schema.external_dns_suffix_search_list
        external_dns_suffix_search_list.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "SecuritySettings"

        _args_schema = cls._args_schema
        _args_schema.sna_lu6point2_transactions_enabled = AAZBoolArg(
            options=["--sna-lu-transactions", "--sna-lu6point2-transactions-enabled"],
            arg_group="SecuritySettings",
            help="Allow SNA LU 6.2 Transactions to managed instance DTC.",
            nullable=True,
        )
        _args_schema.xa_transactions_default_timeout = AAZIntArg(
            options=["--xa-default-timeout", "--xa-transactions-default-timeout"],
            arg_group="SecuritySettings",
            help="Default timeout for XA Transactions (in seconds).",
            nullable=True,
        )
        _args_schema.xa_transactions_enabled = AAZBoolArg(
            options=["--xa-transactions", "--xa-transactions-enabled"],
            arg_group="SecuritySettings",
            help="Allow XA Transactions to managed instance DTC.",
            nullable=True,
        )
        _args_schema.xa_transactions_maximum_timeout = AAZIntArg(
            options=["--xa-max-timeout", "--xa-transactions-maximum-timeout"],
            arg_group="SecuritySettings",
            help="Maximum timeout for XA Transactions (in seconds).",
            nullable=True,
        )

        # define Arg Group "TransactionManagerCommunicationSettings"

        _args_schema = cls._args_schema
        _args_schema.allow_inbound_enabled = AAZBoolArg(
            options=["--allow-inbound-enabled"],
            arg_group="TransactionManagerCommunicationSettings",
            help="Allow Inbound traffic to managed instance DTC.",
            nullable=True,
        )
        _args_schema.allow_outbound_enabled = AAZBoolArg(
            options=["--allow-outbound-enabled"],
            arg_group="TransactionManagerCommunicationSettings",
            help="Allow Outbound traffic of managed instance DTC.",
            nullable=True,
        )
        _args_schema.authentication = AAZStrArg(
            options=["--authentication"],
            arg_group="TransactionManagerCommunicationSettings",
            help="Authentication type of managed instance DTC.",
            nullable=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ManagedInstanceDtcsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.ManagedInstanceDtcsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ManagedInstanceDtcsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/managedInstances/{managedInstanceName}/dtc/{dtcName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "dtcName", "current",
                    required=True,
                ),
                **self.serialize_url_param(
                    "managedInstanceName", self.ctx.args.managed_instance_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-05-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_managed_instance_dtc_read(cls._schema_on_200)

            return cls._schema_on_200

    class ManagedInstanceDtcsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/managedInstances/{managedInstanceName}/dtc/{dtcName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "dtcName", "current",
                    required=True,
                ),
                **self.serialize_url_param(
                    "managedInstanceName", self.ctx.args.managed_instance_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-05-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)

            # Due to a bug in the API, the dtcHostName is returned as the dtcHostNameDnsSuffix property. Also the dtcHostName doesn't exist as a property currently.
            # The new API versions (with a fix) should return dtcHostName property and the correct value for dtcHostNameDnsSuffix.
            # Currently, as a workaround we check if the dtcHostName property exist.
            # If it doesn't exist, we create an additional field "dtcHostName" and display the "dtcHostNameDnsSuffix"
            # from the API response as the host name. Also, the dtcHostNameDnsSuffix is corrected to display the proper value.
            if 'dtcHostName' not in data['properties']:
                data['properties']['dtcHostName'] = data['properties']['dtcHostNameDnsSuffix']

                # Also, we extract the DNS suffix and display it properly.
                data['properties']['dtcHostNameDnsSuffix'] = data['properties']['dtcHostNameDnsSuffix'].split('.', 1)[1]
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_managed_instance_dtc_read(cls._schema_on_200)

            return cls._schema_on_200

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("dtcEnabled", AAZBoolType, ".dtc_enabled")
                properties.set_prop("externalDnsSuffixSearchList", AAZListType, ".external_dns_suffix_search_list")
                properties.set_prop("securitySettings", AAZObjectType)

            external_dns_suffix_search_list = _builder.get(".properties.externalDnsSuffixSearchList")
            if external_dns_suffix_search_list is not None:
                external_dns_suffix_search_list.set_elements(AAZStrType, ".")

            security_settings = _builder.get(".properties.securitySettings")
            if security_settings is not None:
                security_settings.set_prop("snaLu6point2TransactionsEnabled", AAZBoolType, ".sna_lu6point2_transactions_enabled")
                security_settings.set_prop("transactionManagerCommunicationSettings", AAZObjectType)
                security_settings.set_prop("xaTransactionsDefaultTimeout", AAZIntType, ".xa_transactions_default_timeout")
                security_settings.set_prop("xaTransactionsEnabled", AAZBoolType, ".xa_transactions_enabled")
                security_settings.set_prop("xaTransactionsMaximumTimeout", AAZIntType, ".xa_transactions_maximum_timeout")

            transaction_manager_communication_settings = _builder.get(".properties.securitySettings.transactionManagerCommunicationSettings")
            if transaction_manager_communication_settings is not None:
                transaction_manager_communication_settings.set_prop("allowInboundEnabled", AAZBoolType, ".allow_inbound_enabled")
                transaction_manager_communication_settings.set_prop("allowOutboundEnabled", AAZBoolType, ".allow_outbound_enabled")
                transaction_manager_communication_settings.set_prop("authentication", AAZStrType, ".authentication")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_managed_instance_dtc_read = None

    @classmethod
    def _build_schema_managed_instance_dtc_read(cls, _schema):
        if cls._schema_managed_instance_dtc_read is not None:
            _schema.id = cls._schema_managed_instance_dtc_read.id
            _schema.name = cls._schema_managed_instance_dtc_read.name
            _schema.properties = cls._schema_managed_instance_dtc_read.properties
            _schema.type = cls._schema_managed_instance_dtc_read.type
            return

        cls._schema_managed_instance_dtc_read = _schema_managed_instance_dtc_read = AAZObjectType()

        managed_instance_dtc_read = _schema_managed_instance_dtc_read
        managed_instance_dtc_read.id = AAZStrType(
            flags={"read_only": True},
        )
        managed_instance_dtc_read.name = AAZStrType(
            flags={"read_only": True},
        )
        managed_instance_dtc_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        managed_instance_dtc_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_managed_instance_dtc_read.properties
        properties.dtc_enabled = AAZBoolType(
            serialized_name="dtcEnabled",
        )
        properties.dtc_host_name_dns_suffix = AAZStrType(
            serialized_name="dtcHostNameDnsSuffix",
            flags={"read_only": True},
        )
        properties.dtc_host_name = AAZStrType(
            serialized_name="dtcHostName",
            flags={"read_only": True},
        )
        properties.external_dns_suffix_search_list = AAZListType(
            serialized_name="externalDnsSuffixSearchList",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.security_settings = AAZObjectType(
            serialized_name="securitySettings",
        )

        external_dns_suffix_search_list = _schema_managed_instance_dtc_read.properties.external_dns_suffix_search_list
        external_dns_suffix_search_list.Element = AAZStrType()

        security_settings = _schema_managed_instance_dtc_read.properties.security_settings
        security_settings.sna_lu6point2_transactions_enabled = AAZBoolType(
            serialized_name="snaLu6point2TransactionsEnabled",
        )
        security_settings.transaction_manager_communication_settings = AAZObjectType(
            serialized_name="transactionManagerCommunicationSettings",
        )
        security_settings.xa_transactions_default_timeout = AAZIntType(
            serialized_name="xaTransactionsDefaultTimeout",
        )
        security_settings.xa_transactions_enabled = AAZBoolType(
            serialized_name="xaTransactionsEnabled",
        )
        security_settings.xa_transactions_maximum_timeout = AAZIntType(
            serialized_name="xaTransactionsMaximumTimeout",
        )

        transaction_manager_communication_settings = _schema_managed_instance_dtc_read.properties.security_settings.transaction_manager_communication_settings
        transaction_manager_communication_settings.allow_inbound_enabled = AAZBoolType(
            serialized_name="allowInboundEnabled",
        )
        transaction_manager_communication_settings.allow_outbound_enabled = AAZBoolType(
            serialized_name="allowOutboundEnabled",
        )
        transaction_manager_communication_settings.authentication = AAZStrType()

        _schema.id = cls._schema_managed_instance_dtc_read.id
        _schema.name = cls._schema_managed_instance_dtc_read.name
        _schema.properties = cls._schema_managed_instance_dtc_read.properties
        _schema.type = cls._schema_managed_instance_dtc_read.type


__all__ = ["Update"]



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['dms'] = """
type: group
short-summary: Manage Azure Data Migration Service (classic) instances.
"""

helps['dms check-name'] = """
type: command
short-summary: Check if a given DMS instance name is available in a given region as well as the name's validity.
parameters:
  - name: --name -n
    type: string
    short-summary: >
        The service name to check.
examples:
  - name: Check if a given DMS instance name is available in a given region as well as the name's validity. (autogenerated)
    text: az dms check-name --location westus2 --name MyService
    crafted: true
"""

helps['dms check-status'] = """
type: command
short-summary: Perform a health check and return the status of the service and virtual machine size.
examples:
  - name: Perform a health check and return the status of the service and virtual machine size. (autogenerated)
    text: az dms check-status --name MyService --resource-group MyResourceGroup
    crafted: true
"""

helps['dms create'] = """
type: command
short-summary: Create an instance of the Azure Database Migration Service (classic).
parameters:
  - name: --sku-name
    type: string
    short-summary: >
        The name of the CPU SKU on which the service's virtual machine will run. Check the name and the availability of SKUs in your area with "az dms list-skus".
  - name: --subnet
    type: string
    short-summary: >
        The Resource ID of the VNet's Subnet you will use to connect the source and target DBs.
        Use "az network vnet subnet show -h" for help to get your subnet's ID.
examples:
  - name: Create an instance of DMS.
    text: >
        az dms create -l westus -n mydms -g myresourcegroup --sku-name Basic_2vCores --subnet /subscriptions/{vnetSubscriptionId}/resourceGroups/{vnetResourceGroup}/providers/Microsoft.Network/virtualNetworks/{vnetName}/subnets/{subnetName} --tags tagName1=tagValue1 tagWithNoValue
"""

helps['dms delete'] = """
type: command
short-summary: Delete an instance of the Azure Database Migration Service (classic).
parameters:
  - name: --delete-running-tasks
    type: bool
    short-summary: >
        Cancel any running tasks before deleting the service.
examples:
  - name: Delete an instance of the Azure Database Migration Service (classic). (autogenerated)
    text: az dms delete --name MyService --resource-group MyResourceGroup
    crafted: true
"""

helps['dms list'] = """
type: command
short-summary: List the DMS instances within your currently configured subscription (to set this use "az account set"). If provided, only show the instances within a given resource group.
examples:
  - name: List all the instances in your subscription.
    text: >
        az dms list
  - name: List all the instances in a given resource group.
    text: >
        az dms list -g myresourcegroup
"""

helps['dms list-skus'] = """
type: command
short-summary: List the SKUs that are supported by the Azure Database Migration Service (classic).
"""

helps['dms project'] = """
type: group
short-summary: Manage projects for an instance of the Azure Database Migration Service (classic).
"""

helps['dms project check-name'] = """
type: command
short-summary: Check if a given project name is available within a given instance of DMS as well as the name's validity.
parameters:
  - name: --name -n
    type: string
    short-summary: >
        The project name to check.
"""

helps['dms project create'] = """
type: command
short-summary: Create a migration project which can contain multiple tasks.
long-summary: |
  The following project configurations are supported:
    -) source -> target
    1) SQL -> SQLDB
    2) PostgreSQL -> AzureDbForPostgreSQL
    3) MySQL -> AzureDbForMySQL

parameters:
  - name: --source-platform
    type: string
    short-summary: >
        The type of server for the source database. The supported types are: SQL, PostgreSQL, MySQL.
  - name: --target-platform
    type: string
    short-summary: >
        The type of service for the target database. The supported types are: SQLDB, AzureDbForPostgreSQL, AzureDbForMySQL.
examples:
  - name: Create a SQL to SQLDB project for a DMS instance.
    text: >
        az dms project create -l westus -n sqlproject -g myresourcegroup --service-name mydms --source-platform SQL --target-platform SQLDB --tags tagName1=tagValue1 tagWithNoValue
  - name: Create a PostgreSql to AzureDbForPostgreSql project for a DMS instance.
    text: >
        az dms project create -l westus -n pgproject -g myresourcegroup --service-name mydms --source-platform PostgreSQL --target-platform AzureDbForPostgreSQL --tags tagName1=tagValue1 tagWithNoValue
  - name: Create a MySQL to AzureDbForMySQL project for a DMS instance.
    text: >
        az dms project create -l westus -n mysqlproject -g myresourcegroup --service-name mydms --source-platform MySQL --target-platform AzureDbForMySQL --tags tagName1=tagValue1 tagWithNoValue
"""

helps['dms project delete'] = """
type: command
short-summary: Delete a project.
parameters:
  - name: --delete-running-tasks
    type: bool
    short-summary: >
        Cancel any running tasks before deleting the project.
examples:
  - name: Delete a project. (autogenerated)
    text: az dms project delete --name MyProject --resource-group MyResourceGroup --service-name MyService
    crafted: true
"""

helps['dms project list'] = """
type: command
short-summary: List the projects within an instance of DMS.
examples:
  - name: List the projects within an instance of DMS. (autogenerated)
    text: az dms project list --resource-group MyResourceGroup --service-name MyService
    crafted: true
"""

helps['dms project show'] = """
type: command
short-summary: Show the details of a migration project.
examples:
  - name: Show the details of a migration project. (autogenerated)
    text: az dms project show --name MyProject --resource-group MyResourceGroup --service-name MyService
    crafted: true
"""

helps['dms project task'] = """
type: group
short-summary: Manage tasks for a Azure Database Migration Service (classic) instance's project.
"""

helps['dms project task cancel'] = """
type: command
short-summary: Cancel a task if it's currently queued or running.
"""

helps['dms project task check-name'] = """
type: command
short-summary: Check if a given task name is available within a given instance of DMS as well as the name's validity.
parameters:
  - name: --name -n
    type: string
    short-summary: >
        The task name to check.
"""

helps['dms project task create'] = """
type: command
short-summary: Create and start a migration task.
long-summary: |
  The following task configurations are supported:
    -) source -> target :: task type
    1) SQL -> SQLDB :: OfflineMigration
    2) PostgreSQL -> AzureDbForPostgreSql :: OnlineMigration
    3) MySQL -> AzureDbForMySQL :: OfflineMigration
parameters:
  - name: --task-type
    type: string
    short-summary: >
        The type of data movement the task will support. The supported types are: OnlineMigration, OfflineMigration. If not provided, will default to OfflineMigration for SQL, MySQL and OnlineMigration for PostgreSQL.
  - name: --database-options-json
    type: string
    short-summary: >
        Database and table information. This can be either a JSON-formatted string or the location to a file containing the JSON object. See example below for the format.
    long-summary: >
        For SQL we support per table migrations. To use this, specify the tables names in the 'table_map' as below.
        You can also set the source as read only.
            [
                {
                    "name": "source database",
                    "target_database_name": "target database",
                    "make_source_db_read_only": false|true,
                    "table_map": {
                        "schema.SourceTableName1": "schema.TargetTableName1",
                        "schema.SourceTableName2": "schema.TargetTableName2",
                        ...n
                    }
                },
                ...n
            ]

        For PostgreSQL, the format of the database options JSON object.
            [
                {
                    "name": "source database",
                    "target_database_name": "target database",
                    // Used for manipulating the underlying migration engine.
                    // Only provide if instructed to do so or if you really know what you are doing.
                    "migrationSetting": {
                        "setting1": "value1",
                        ...n
                    },
                    // Used for manipulating the underlying migration engine.
                    // Only provide if instructed to do so or if you really know what you are doing.
                    "sourceSetting": {
                        "setting1": "value1",
                        ...n
                    },
                    // Used for manipulating the underlying migration engine.
                    // Only provide if instructed to do so or if you really know what you are doing.
                    "targetSetting": {
                        "setting1": "value1",
                        ...n
                    },
                    // Optional parameter to list tables that you want included in the migration.
                    "selectedTables": [
                        "schemaName1.tableName1",
                        ...n
                    ]
                },
                ...n
            ]

        For MySQL, the format of the database options JSON object.
        {
            // Details of mapped schemas that needs to be migrated. Multiple schemas can be migrated at a time.
            "selected_databases":[
                // database/schema 1 details
                {
                  "name": "sourceSchema1",
                  "target_database_name": "targetSchema1",
                  // Table mapping from source to target schemas [Optional]
                  // Don't add it if all tables of this database needs to be migrated
                  "table_map": {
                        "sourceSchema1.table1": "targetSchema1.table1",
                        "sourceSchema1.table2": "targetSchema1.table2",
                        "sourceSchema1.table3": "targetSchema1.table3",
                        ..n
                    }
                    // the below items are only necessary for selective schema migration
                    // optional, migrates schema for the following tables
                    'tables_to_migrate_schema': {
                        "sourceSchema1.table2": "targetSchema1.table2",
                        "sourceSchema1.table3": "targetSchema1.table3"
                    },
                    // optional, migrates the enumerated views
                    'selected_views': [
                        'sourceSchema1.view1'
                    ],
                    // optional, migrates the enumerated triggers
                    'selected_triggers': [
                        'sourceSchema1.on_table1_updated'
                    ],
                    // optional, migrates the enumerated routines
                    'selected_routines': [
                        'sourceSchema1.build_report'
                    ],
                    // optional, migrates the enumerated events
                    'selected_events': [
                        'sourceSchema1.nightly_maintenance'
                    ]
                },
                ...n
            ],

            // Used for manipulating the underlying migration engine. [Optional]
            // Only provide if instructed to do so or if you really know what you are doing.
            "migration_level_settings": {
                // Optional setting that configures the maximum number of parallel reads on tables located on the source database.
                "DesiredRangesCount": "4",
                // Optional setting that configures that size of the largest batch that will be committed to the target server.
                "MaxBatchSizeKb": "4096",
                // Optional setting that configures the minimum number of rows in each batch written to the target.
                "MinBatchRows": null,
                // Optional setting that configures the number of databases that will be prepared for migration in parallel.
                "PrepareDatabaseForBulkImportTaskCount": null,
                // Optional setting that configures the number of tables that will be prepared for migration in parallel.
                "PrepareTableForBulkImportTaskCount": null,
                // Optional setting that configures the number of threads available to read ranges on the source.
                "QueryTableDataRangeTaskCount": "8",
                // Optional setting that configures the number of threads available to write batches to the target.
                "WriteDataRangeBatchTaskCount": "12",
                // Optional setting that configures how much memory will be used to cache batches in memory before reads on the source are throttled.
                "MaxBatchCacheSizeMb": null,
                // Optional setting that configures the amount of available memory at which point reads on the source will be throttled.
                "ThrottleQueryTableDataRangeTaskAtAvailableMemoryMb": null,
                // Optional setting that configures the number of batches cached in memory that will trigger read throttling on the source.
                "ThrottleQueryTableDataRangeTaskAtBatchCount": 36,
                // Optional setting that configures the delay between updates of result objects in Azure Table Storage.
                "DelayProgressUpdatesInStorageInterval": "00:00:30",
            },
            // Optional setting to set the source server read only.
            "make_source_server_read_only": "true|false",
            // Optional setting to enable consistent backup. True by default for the sync migration, and false otherwise.
            "enable_consistent_backup": "true|false",
            // Optional. If true, all view definitions will be migrated in the selected databases.
            "migrate_all_views": "true|false",
            // Optional. If true, all trigger definitions will be migrated in the selected databases.
            "migrate_all_triggers": "true|false",
            // Optional. If true, all event definitions will be migrated in the selected databases.
            "migrate_all_events": "true|false",
            // Optional. If true, all stored proc definitions will be migrated in the selected databases.
            "migrate_all_routines": "true|false",
            // Optional. If true, all table's schemas will be migrated.
            "migrate_all_tables_schema": "true|false",
            // Optional. If true, all users/grants will be migrated.
            "migrate_user_system_tables": "true|false",
            // Binlog position to start the migration from. Only applicable for the ReplicateChanges migration.
            "binLogInfo": {
                "filename": "binlog.0004523",
                "position": 283287
            }
        }

  - name: --source-connection-json
    type: string
    short-summary: >
        The connection information to the source server. This can be either a JSON-formatted string or the location to a file containing the JSON object. See examples below for the format.
    long-summary: |
      The format of the connection JSON object for SQL connections.
        {
            "userName": "user name",    // if this is missing or null, you will be prompted
            "password": null,           // if this is missing or null (highly recommended) you will be prompted
            "dataSource": "server name[,port]",
            "authentication": "SqlAuthentication|WindowsAuthentication",
            "encryptConnection": true,      // highly recommended to leave as true
            "trustServerCertificate": false  // highly recommended to leave as false
        }

      The format of the connection JSON object for PostgreSQL connections.
        {
            "userName": "user name",    // if this is missing or null, you will be prompted
            "password": null,           // if this is missing or null (highly recommended) you will be prompted
            "serverName": "server name",
            "databaseName": "database name", // if this is missing, it will default to the 'postgres' database
            "port": 5432,                // if this is missing, it will default to 5432
            "encryptConnection": true,      // highly recommended to leave as true
            "trustServerCertificate": false  // highly recommended to leave as false
        }

      The format of the connection JSON object for MySQL connections.
        {
            "userName": "user name",    // if this is missing or null, you will be prompted
            "password": null,           // if this is missing or null (highly recommended) you will be prompted
            "serverName": "server name",
            "port": 3306                // if this is missing, it will default to 3306
        }
  - name: --target-connection-json
    type: string
    short-summary: >
        The connection information to the target server. This can be either a JSON-formatted string or the location to a file containing the JSON object. See 'source-connection-json' for examples of connection formats.
  - name: --enable-data-integrity-validation
    type: bool
    short-summary: >
        For SQL only. Whether to perform a checksum based data integrity validation between source and target for the selected database and tables.
  - name: --enable-query-analysis-validation
    type: bool
    short-summary: >
        For SQL only. Whether to perform a quick and intelligent query analysis by retrieving queries from the source database and
        executing them in the target. The result will have execution statistics for executions in source and target databases
        for the extracted queries.
  - name: --enable-schema-validation
    type: bool
    short-summary: >
        For SQL only. Whether to compare the schema information between source and target.
examples:
  - name: Create and start an offline SQL migration task. For a SQL migration, this will perform no validation checks.
    text: >
        az dms project task create --task-type OfflineMigration --database-options-json "C:\\CLI Files\\databaseOptions.json" -n mytask --project-name myproject -g myresourcegroup --service-name mydms --source-connection-json "{'dataSource': 'myserver', 'authentication': 'SqlAuthentication', 'encryptConnection': 'true', 'trustServerCertificate': 'true'}" --target-connection-json "C:\\CLI Files\\targetConnection.json"
  - name: Create and start a SQL task which performs all validation checks.
    text: >
        az dms project task create --task-type OfflineMigration --database-options-json "C:\\CLI Files\\databaseOptions.json" -n mytask --project-name myproject -g myresourcegroup --service-name mydms --source-connection-json "C:\\CLI Files\\sourceConnection.json" --target-connection-json "C:\\CLI Files\\targetConnection.json" --enable-data-integrity-validation --enable-query-analysis-validation --enable-schema-validation
"""

helps['dms project task delete'] = """
type: command
short-summary: Delete a migration task.
parameters:
  - name: --delete-running-tasks
    type: bool
    short-summary: >
        If the task is currently running, cancel the task before deleting the project.
examples:
  - name: Delete a migration task. (autogenerated)
    text: az dms project task delete --name MyTask --project-name MyProject --resource-group MyResourceGroup --service-name MyService
    crafted: true
"""

helps['dms project task list'] = """
type: command
short-summary: List the tasks within a project. Some tasks may have a status of Unknown, which indicates that an error occurred while querying the status of that task.
parameters:
  - name: --task-type
    type: string
    short-summary: >
        Filters the list by the type of task. For the list of possible types see "az dms check-status".
examples:
  - name: List all tasks within a project.
    text: >
        az dms project task list --project-name myproject -g myresourcegroup --service-name mydms
  - name: List only the SQL to SQL migration tasks within a project.
    text: >
        az dms project task list --project-name myproject -g myresourcegroup --service-name mydms --task-type Migrate.SqlServer.SqlDb
"""

helps['dms project task show'] = """
type: command
short-summary: Show the details of a migration task. Use the "--expand" to get more details.
parameters:
  - name: --expand
    type: string
    short-summary: >
        Expand the response to provide more details. Use with "command" to see more details of the task.
        Use with "output" to see the results of the task's migration.
examples:
  - name: Show the details of a migration task. Use the "--expand" to get more details. (autogenerated)
    text: az dms project task show --name MyTask --project-name MyProject --resource-group MyResourceGroup --service-name MyService
    crafted: true
"""

helps['dms project task cutover'] = """
type: command
short-summary: For an online migration task, complete the migration by performing a cutover.
long-summary: |
  To see the result of the request, please use the 'task show' command:
      az dms project task show ... --expand command

parameters:
  - name: --object-name
    type: string
    short-summary: >
      The name of the database on the source you wish to cutover.
"""

helps['dms show'] = """
type: command
short-summary: Show the details for an instance of the Azure Database Migration Service (classic).
"""

helps['dms start'] = """
type: command
short-summary: Start an instance of the Azure Database Migration Service (classic). It can then be used to run data migrations.
examples:
  - name: Start an instance of the Azure Database Migration Service (classic). It can then be used to run data migrations. (autogenerated)
    text: az dms start --name MyService --resource-group MyResourceGroup
    crafted: true
"""

helps['dms stop'] = """
type: command
short-summary: Stop an instance of the Azure Database Migration Service (classic). While stopped, it can't be used to run data migrations and the owner won't be billed.
"""

helps['dms wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the DMS instance is met.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import ipaddress


from azure.cli.core.azclierror import (InvalidArgumentValueError, ArgumentUsageError, RequiredArgumentMissingError,
                                       ResourceNotFoundError, ValidationError, MutuallyExclusiveArgumentError)
from azure.cli.core.commands.client_factory import get_mgmt_service_client, get_subscription_id
from azure.cli.core.profiles import ResourceType
from azure.cli.core.commands.validators import validate_tags

from knack.log import get_logger
from msrestazure.tools import is_valid_resource_id, parse_resource_id

from ._appservice_utils import _generic_site_operation
from ._client_factory import web_client_factory
from .utils import (_normalize_sku, get_sku_tier, _normalize_location, get_resource_name_and_group,
                    get_resource_if_exists, is_functionapp, is_logicapp, is_webapp)

logger = get_logger(__name__)


def validate_and_convert_to_int(flag, val):
    try:
        return int(val)
    except ValueError:
        raise ArgumentUsageError("{} is expected to have an int value.".format(flag))


def validate_range_of_int_flag(flag_name, value, min_val, max_val):
    value = validate_and_convert_to_int(flag_name, value)
    if min_val > value or value > max_val:
        raise ArgumentUsageError("Usage error: {} is expected to be between {} and {} (inclusive)".format(flag_name,
                                                                                                          min_val,
                                                                                                          max_val))
    return value


def validate_timeout_value(namespace):
    """Validates that zip deployment timeout is set to a reasonable min value"""
    if isinstance(namespace.timeout, int):
        if namespace.timeout <= 29:
            raise ArgumentUsageError('--timeout value should be a positive value in seconds and should be at least 30')


def validate_site_create(cmd, namespace):
    """Validate the SiteName that is being used to create is available
    This API requires that the RG is already created"""
    client = web_client_factory(cmd.cli_ctx)
    if isinstance(namespace.name, str) and isinstance(namespace.resource_group_name, str) \
            and isinstance(namespace.plan, str):
        resource_group_name = namespace.resource_group_name
        plan = namespace.plan
        if is_valid_resource_id(plan):
            parsed_result = parse_resource_id(plan)
            plan_info = client.app_service_plans.get(parsed_result['resource_group'], parsed_result['name'])
        else:
            plan_info = client.app_service_plans.get(resource_group_name, plan)
        if not plan_info:
            raise ResourceNotFoundError("The plan '{}' doesn't exist in the resource group '{}'".format(
                plan, resource_group_name))
        # verify that the name is available for create
        validation_payload = {
            "name": namespace.name,
            "type": "Microsoft.Web/sites",
            "location": plan_info.location,
            "properties": {
                "serverfarmId": plan_info.id
            }
        }
        validation = client.validate(resource_group_name, validation_payload)
        if validation.status.lower() == "failure" and validation.error.code != 'SiteAlreadyExists':
            raise ValidationError(validation.error.message)


def validate_ase_create(cmd, namespace):
    # Validate the ASE Name availability
    client = web_client_factory(cmd.cli_ctx)
    resource_type = 'Microsoft.Web/hostingEnvironments'
    if isinstance(namespace.name, str):
        name_validation = client.check_name_availability(namespace.name, resource_type)
        if not name_validation.name_available:
            raise ValidationError(name_validation.message)


def _validate_asp_sku(sku, app_service_environment, zone_redundant):
    supported_skus = ['PREMIUMV2', 'PREMIUMV3', 'PREMIUMMV3', 'PREMIUM0V3', 'ISOLATEDV2']
    if zone_redundant and get_sku_tier(sku).upper() not in supported_skus:
        raise ValidationError("Zone redundancy cannot be enabled for sku {}".format(sku))
    # Isolated SKU is supported only for ASE
    if sku.upper() in ['I1', 'I2', 'I3', 'I1V2', 'I2V2', 'I3V2', 'I4V2', 'I5V2', 'I6V2']:
        if not app_service_environment:
            raise ValidationError("The pricing tier 'Isolated' is not allowed for this app service plan. "
                                  "Use this link to learn more: "
                                  "https://docs.microsoft.com/azure/app-service/overview-hosting-plans")
    else:
        if app_service_environment:
            raise ValidationError("Only pricing tier 'Isolated' is allowed in this app service plan. Use this link to "
                                  "learn more: https://docs.microsoft.com/azure/app-service/overview-hosting-plans")


def validate_asp_create(namespace):
    validate_tags(namespace)
    sku = _normalize_sku(namespace.sku)
    _validate_asp_sku(sku, namespace.app_service_environment, namespace.zone_redundant)
    if namespace.is_linux and namespace.hyper_v:
        raise MutuallyExclusiveArgumentError('Usage error: --is-linux and --hyper-v cannot be used together.')


def validate_functionapp_asp_create(namespace):
    validate_tags(namespace)
    sku = _normalize_sku(namespace.sku)
    tier = get_sku_tier(sku)
    _validate_asp_sku(sku=sku, app_service_environment=None, zone_redundant=namespace.zone_redundant)
    if namespace.max_burst is not None:
        if tier.lower() != "elasticpremium":
            raise ArgumentUsageError("--max-burst is only supported for Elastic Premium (EP) plans")


def validate_app_exists(cmd, namespace):
    app = namespace.name
    resource_group_name = namespace.resource_group_name
    slot = namespace.slot
    app = _generic_site_operation(cmd.cli_ctx, resource_group_name, app, 'get', slot)
    if not app:
        raise ResourceNotFoundError("'{}' app not found in ResourceGroup '{}'".format(app, resource_group_name))


def validate_add_vnet(cmd, namespace):
    from azure.core.exceptions import ResourceNotFoundError as ResNotFoundError

    resource_group_name = namespace.resource_group_name
    from azure.cli.command_modules.network._client_factory import network_client_factory

    vnet_identifier = namespace.vnet
    name = namespace.name
    slot = namespace.slot

    if is_valid_resource_id(vnet_identifier):
        current_sub_id = get_subscription_id(cmd.cli_ctx)
        parsed_vnet = parse_resource_id(vnet_identifier)

        vnet_sub_id = parsed_vnet['subscription']
        vnet_group = parsed_vnet['resource_group']
        vnet_name = parsed_vnet['name']

        cmd.cli_ctx.data['subscription_id'] = vnet_sub_id
        vnet_client = network_client_factory(cmd.cli_ctx)
        vnet_loc = vnet_client.virtual_networks.get(resource_group_name=vnet_group,
                                                    virtual_network_name=vnet_name).location
        cmd.cli_ctx.data['subscription_id'] = current_sub_id
    else:
        vnet_client = network_client_factory(cmd.cli_ctx)

        try:
            vnet_loc = vnet_client.virtual_networks.get(resource_group_name=namespace.resource_group_name,
                                                        virtual_network_name=vnet_identifier).location
        except ResNotFoundError:
            vnets = vnet_client.virtual_networks.list_all()
            vnet_loc = ''
            for v in vnets:
                if vnet_identifier == v.name:
                    vnet_loc = v.location
                    break

    if not vnet_loc:
        # Fall back to back end validation
        logger.warning("Failed to fetch vnet. Skipping location validation.")
        return

    webapp = _generic_site_operation(cmd.cli_ctx, resource_group_name, name, 'get', slot)

    webapp_loc = _normalize_location(cmd, webapp.location)
    vnet_loc = _normalize_location(cmd, vnet_loc)

    if vnet_loc != webapp_loc:
        raise ValidationError("The app and the vnet resources are in different locations. "
                              "Cannot integrate a regional VNET to an app in a different region")


def validate_front_end_scale_factor(namespace):
    if namespace.front_end_scale_factor:
        min_scale_factor = 5
        max_scale_factor = 15
        scale_error_text = "Frontend Scale Factor '{}' is invalid. Must be between {} and {}"
        scale_factor = namespace.front_end_scale_factor
        if scale_factor < min_scale_factor or scale_factor > max_scale_factor:
            raise ValidationError(scale_error_text.format(scale_factor, min_scale_factor, max_scale_factor))


def validate_ip_address(cmd, namespace):
    if namespace.ip_address is not None:
        _validate_ip_address_format(namespace)
        # For prevention of adding the duplicate IPs.
        if 'add' in cmd.name:
            _validate_ip_address_existence(cmd, namespace)


def validate_onedeploy_params(namespace):
    if namespace.src_path and namespace.src_url:
        raise MutuallyExclusiveArgumentError('Only one of --src-path and --src-url can be specified')

    if not namespace.src_path and not namespace.src_url:
        raise RequiredArgumentMissingError('Either of --src-path or --src-url must be specified')

    if namespace.src_url and not namespace.artifact_type:
        raise RequiredArgumentMissingError('Deployment type is mandatory when deploying from URLs. Use --type')


def _validate_ip_address_format(namespace):
    if namespace.ip_address is not None:
        input_value = namespace.ip_address
        if ' ' in input_value:
            raise InvalidArgumentValueError("Spaces not allowed: '{}' ".format(input_value))
        input_ips = input_value.split(',')
        if len(input_ips) > 8:
            raise InvalidArgumentValueError('Maximum 8 IP addresses are allowed per rule.')
        validated_ips = ''
        for ip in input_ips:
            # Use ipaddress library to validate ip network format
            ip_obj = ipaddress.ip_network(ip, False)
            validated_ips += str(ip_obj) + ','
        namespace.ip_address = validated_ips[:-1]


def _validate_ip_address_existence(cmd, namespace):
    resource_group_name = namespace.resource_group_name
    name = namespace.name
    slot = namespace.slot
    scm_site = namespace.scm_site
    configs = _generic_site_operation(cmd.cli_ctx, resource_group_name, name, 'get_configuration', slot)
    access_rules = configs.scm_ip_security_restrictions if scm_site else configs.ip_security_restrictions
    ip_exists = [(lambda x: x.ip_address == namespace.ip_address)(x) for x in access_rules]
    if True in ip_exists:
        raise ArgumentUsageError('IP address: ' + namespace.ip_address + ' already exists. '
                                 'Cannot add duplicate IP address values.')


def validate_service_tag(cmd, namespace):
    if namespace.service_tag is not None:
        _validate_service_tag_format(cmd, namespace)
        # For prevention of adding the duplicate IPs.
        if 'add' in cmd.name:
            _validate_service_tag_existence(cmd, namespace)


def _validate_service_tag_format(cmd, namespace):
    if namespace.service_tag is not None:
        input_value = namespace.service_tag
        if ' ' in input_value:
            raise InvalidArgumentValueError("Spaces not allowed: '{}' ".format(input_value))
        input_tags = input_value.split(',')
        if len(input_tags) > 8:
            raise InvalidArgumentValueError('Maximum 8 service tags are allowed per rule.')
        network_client = get_mgmt_service_client(cmd.cli_ctx, ResourceType.MGMT_NETWORK)
        resource_group_name = namespace.resource_group_name
        name = namespace.name
        webapp = _generic_site_operation(cmd.cli_ctx, resource_group_name, name, 'get')
        service_tag_full_list = network_client.service_tags.list(webapp.location).values
        if service_tag_full_list is None:
            logger.warning('Not able to get full Service Tag list. Cannot validate Service Tag.')
            return
        for tag in input_tags:
            valid_tag = False
            for tag_full_list in service_tag_full_list:
                if tag.lower() == tag_full_list.name.lower():
                    valid_tag = True
                    continue
            if not valid_tag:
                raise InvalidArgumentValueError('Unknown Service Tag: ' + tag)


def _validate_service_tag_existence(cmd, namespace):
    resource_group_name = namespace.resource_group_name
    name = namespace.name
    slot = namespace.slot
    scm_site = namespace.scm_site
    input_tag_value = namespace.service_tag.replace(' ', '')
    configs = _generic_site_operation(cmd.cli_ctx, resource_group_name, name, 'get_configuration', slot)
    access_rules = configs.scm_ip_security_restrictions if scm_site else configs.ip_security_restrictions
    for rule in access_rules:
        if rule.ip_address and rule.ip_address.lower() == input_tag_value.lower():
            raise ArgumentUsageError('Service Tag: ' + namespace.service_tag + ' already exists. '
                                     'Cannot add duplicate Service Tag values.')


def validate_public_cloud(cmd):
    from azure.cli.core.cloud import AZURE_PUBLIC_CLOUD
    if cmd.cli_ctx.cloud.name != AZURE_PUBLIC_CLOUD.name:
        raise ValidationError('This command is not yet supported on soveriegn clouds.')


def validate_staticsite_sku(cmd, namespace):
    from azure.mgmt.web import WebSiteManagementClient
    client = get_mgmt_service_client(cmd.cli_ctx, WebSiteManagementClient).static_sites
    sku_name = client.get_static_site(namespace.resource_group_name, namespace.name).sku.name
    if sku_name.lower() != "standard":
        raise ValidationError("Invalid SKU: '{}'. Staticwebapp must have 'Standard' SKU".format(sku_name))


def validate_staticsite_link_function(cmd, namespace):
    from azure.mgmt.web import WebSiteManagementClient
    validate_staticsite_sku(cmd, namespace)

    if not is_valid_resource_id(namespace.function_resource_id):
        raise ArgumentUsageError("--function-resource-id must specify a function resource ID. "
                                 "To get resource ID, use the following commmand, inserting the function "
                                 "group/name as needed: \n"
                                 "az functionapp show --resource-group \"[FUNCTION_RESOURCE_GROUP]\" "
                                 "--name \"[FUNCTION_NAME]\" --query id ")

    client = get_mgmt_service_client(cmd.cli_ctx, WebSiteManagementClient, api_version="2020-12-01").static_sites
    functions = client.get_user_provided_function_apps_for_static_site(
        name=namespace.name, resource_group_name=namespace.resource_group_name)
    if list(functions):
        raise ValidationError("Cannot have more than one user provided function app associated with a Static Web App")


# TODO consider combining with validate_add_vnet
def validate_vnet_integration(cmd, namespace):
    validate_tags(namespace)
    if namespace.subnet or namespace.vnet:
        if not namespace.subnet:
            raise ArgumentUsageError("Cannot use --vnet without --subnet")
        if not is_valid_resource_id(namespace.subnet) and not namespace.vnet:
            raise ArgumentUsageError("Must either specify subnet by resource ID or include --vnet argument")

        client = web_client_factory(cmd.cli_ctx)
        if is_valid_resource_id(namespace.plan):
            parse_result = parse_resource_id(namespace.plan)
            plan_info = client.app_service_plans.get(parse_result['resource_group'], parse_result['name'])
        else:
            plan_info = client.app_service_plans.get(name=namespace.plan,
                                                     resource_group_name=namespace.resource_group_name)

        sku_name = plan_info.sku.name
        disallowed_skus = {'FREE', 'SHARED', 'PremiumContainer', 'Isolated', 'IsolatedV2'}
        if get_sku_tier(sku_name) in disallowed_skus:
            raise ArgumentUsageError("App Service Plan has invalid sku for vnet integration: {}."
                                     "Plan sku cannot be one of: {}. "
                                     "Please run 'az appservice plan create -h' "
                                     "to see all available App Service Plan SKUs ".format(sku_name, disallowed_skus))


def _validate_ase_exists(client, ase_name, ase_rg):
    extant_ase = get_resource_if_exists(client.app_service_environments,
                                        resource_group_name=ase_rg, name=ase_name)
    if extant_ase is None:
        raise ValidationError("App Service Environment {} does not exist.".format(ase_name))


# if the ASP exists, validate that it is in the ASE
def _validate_plan_in_ase(client, plan_name, plan_rg, ase_id):
    if plan_name is not None:
        plan_info = get_resource_if_exists(client.app_service_plans,
                                           resource_group_name=plan_rg, name=plan_name)
        if plan_info is not None:
            plan_hosting_env = plan_info.hosting_environment_profile

            if not plan_hosting_env or plan_hosting_env.id != ase_id:
                raise ValidationError("Plan {} already exists and is not in the "
                                      "app service environment.".format(plan_name))


def _validate_ase_is_v3(ase):
    if ase.kind.upper() != "ASEV3":
        raise ValidationError("Only V3 App Service Environments supported")


def _validate_ase_not_ilb(ase):
    if (ase.internal_load_balancing_mode != 0) and (ase.internal_load_balancing_mode != "None"):
        raise ValidationError("Internal Load Balancer (ILB) App Service Environments not supported")


def validate_webapp_up(cmd, namespace):
    if namespace.runtime and namespace.html:
        raise MutuallyExclusiveArgumentError('Conflicting parameters: cannot have both --runtime and --html specified.')

    client = web_client_factory(cmd.cli_ctx)
    if namespace.app_service_environment:
        ase_name, ase_rg, ase_id = get_resource_name_and_group(cmd, namespace.app_service_environment,
                                                               namespace.resource_group_name,
                                                               namespace="Microsoft.Web",
                                                               type="hostingEnvironments")
        _validate_ase_exists(client, ase_name, ase_rg)
        _validate_plan_in_ase(client, namespace.plan, namespace.resource_group_name, ase_id)

        ase = client.app_service_environments.get(resource_group_name=ase_rg, name=ase_name)
        _validate_ase_is_v3(ase)
        _validate_ase_not_ilb(ase)


def _get_app_name(namespace):
    if hasattr(namespace, "name"):
        return namespace.name
    if hasattr(namespace, "webapp"):
        return namespace.webapp
    return None


def validate_app_is_webapp(cmd, namespace):
    client = web_client_factory(cmd.cli_ctx)
    name = _get_app_name(namespace)
    rg = namespace.resource_group
    app = get_resource_if_exists(client.web_apps, name=name, resource_group_name=rg)
    if is_functionapp(app):
        raise ValidationError(f"App '{name}' in group '{rg}' is a function app.")
    if is_logicapp(app):
        raise ValidationError(f"App '{name}' in group '{rg}' is a logic app.")


def validate_app_is_functionapp(cmd, namespace):
    client = web_client_factory(cmd.cli_ctx)
    name = _get_app_name(namespace)
    rg = namespace.resource_group
    app = get_resource_if_exists(client.web_apps, name=name, resource_group_name=rg)
    if is_logicapp(app):
        raise ValidationError(f"App '{name}' in group '{rg}' is a logic app.")
    if is_webapp(app):
        raise ValidationError(f"App '{name}' in group '{rg}' is a web app.")
